
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: CreateFromMixins, GRAY_FONT_COLOR, HIGHLIGHT_FONT_COLOR, SecondsFormatter, SecondsFormatterMixin, string

local abs = abs
local C_ChallengeMode = C_ChallengeMode
local GetWorldElapsedTime = GetWorldElapsedTime
local next = next
local select = select
local setmetatable = setmetatable

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(SixUI) -- AddOn, Border, Constants, Database
local M = A:NewModule("Display MythicPlus")
local O = LibStub("LibDataBroker-1.1"):NewDataObject("SixUI MythicPlus", {
	description = "Shows while you're in Mythic+ dungeon.",
	type = "progress bar",
	label = UNKNOWN,
	level = UNKNOWN,
	text = UNKNOWN,
	value = "0",
	projectedValue = "0",
	suffix = "%",
	visible = false,
})

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

M.Rating = {
	Gold = "gold",
	Silver = "silver",
	Bronze = "bronze",
	Untimed = "untimed",
}

M.RatingColor = {
	[M.Rating.Gold] = C.GOLD_RATING_COLOR,
	[M.Rating.Silver] = C.SILVER_RATING_COLOR,
	[M.Rating.Bronze] = C.BRONZE_RATING_COLOR,
	[M.Rating.Untimed] = C.UNTIMED_RATING_COLOR,
}

M.RatingText = {
	[M.Rating.Gold] = "+3",
	[M.Rating.Silver] = "+2",
	[M.Rating.Bronze] = "+1",
	[M.Rating.Untimed] = CHALLENGE_MODE_KEYSTONE_DEPLETED,
}

M.TimersMetatable = {
	__index = function(timers, elapsedTime)
		for _, ratingID in next, {M.Rating.Gold, M.Rating.Silver, M.Rating.Bronze} do
			if elapsedTime < timers[ratingID] then
				return ratingID
			end
		end

		return M.Rating.Untimed
	end
}

function M.GetMythicPlusInfo()
	local mapID = C_ChallengeMode.GetActiveChallengeMapID()
	if (not mapID or mapID == 0) then return end

	local name, _, timeLimit = C_ChallengeMode.GetMapUIInfo(mapID)
	local level = C_ChallengeMode.GetActiveKeystoneInfo()
	local elapsedTime = select(2, GetWorldElapsedTime(1))
	local time = abs(timeLimit - elapsedTime)
	local timeText = M.timeFormatter:Format(time, false, true)

	return {
		name = name,
		level = level,
		timeLimit = timeLimit,
		elapsedTime = elapsedTime,
		time = time,
		timeText = timeText,
	}
end

function M.GetRatingInfo(timeLimit, elapsedTime)
	local timers = M.GetTimers(timeLimit)
	local ratingID = timers[elapsedTime]

	return {
		id = ratingID,
		timeLimit = (ratingID == M.Rating.Untimed and elapsedTime or timers[ratingID]),
		color = M.RatingColor[ratingID],
		text = M.RatingText[ratingID],
	}
end

function M.GetTimers(timeLimit)
	return setmetatable({
		[M.Rating.Gold] = timeLimit * 0.6,
		[M.Rating.Silver] = timeLimit * 0.8,
		[M.Rating.Bronze] = timeLimit,
	}, M.TimersMetatable)
end

function M:OnInitialize()
	self.timeFormatter = CreateFromMixins(SecondsFormatterMixin)
	self.timeFormatter:Init(1, SecondsFormatter.Abbreviation.Truncate, false, true, true)
end

function M:OnEnable()
	self:RegisterEvent("CHALLENGE_MODE_COMPLETED", "Update")
	self:RegisterEvent("CHALLENGE_MODE_START", "Update")
	self:RegisterMessage("SIXUI_CONFIG_UPDATE", "OnConfigUpdate")
	self:Update()
end

function M:OnConfigUpdate(event, key, ...)
	local c, g, k = string.split(".", key)

	if c == "color" and g == "rating" then
		self:Update()
	end
end

function M:Update()
	local mythicPlusInfo = self.GetMythicPlusInfo()

	if (not mythicPlusInfo) then
		O.visible = false
		return
	end

	local ratingInfo = self.GetRatingInfo(mythicPlusInfo.timeLimit, mythicPlusInfo.elapsedTime)

	O.barR, O.barG, O.barB = ratingInfo.color.r, ratingInfo.color.g, ratingInfo.color.b
	O.label = A.FormatInstanceName(mythicPlusInfo.name, nil, mythicPlusInfo.level)
	O.text = ratingInfo.color:WrapTextInColorCode(mythicPlusInfo.timeText)
	O.value = mythicPlusInfo.elapsedTime / mythicPlusInfo.timeLimit * 100
	O.projectedValue = ratingInfo.timeLimit / mythicPlusInfo.timeLimit * 100
	O.visible = true

	self:ScheduleTimer("Update", 1)
end

-------------------------------------------------------------------------------
-- DataObject
-------------------------------------------------------------------------------

function O.OnTooltipShow(tooltip)
	local mythicPlusInfo = M.GetMythicPlusInfo()
	if (not mythicPlusInfo) then return end

	tooltip:SetText(O.label)

	for ratingID, timeLimit in next, M.GetTimers(mythicPlusInfo.timeLimit) do
		local ratingText = M.RatingText[ratingID]
		local ratingColor = M.RatingColor[ratingID]
		local timeText = M.timeFormatter:Format(timeLimit - mythicPlusInfo.elapsedTime, false, true)
		local timeColor = (mythicPlusInfo.elapsedTime < timeLimit) and HIGHLIGHT_FONT_COLOR or GRAY_FONT_COLOR

		tooltip:AddDoubleLine(ratingColor:WrapTextInColorCode(ratingText), timeColor:WrapTextInColorCode(timeText))
	end
end
